<?php

$accounts=30;
$groupName='Course Students';

//define class autoloader
spl_autoload_register(function($className){
    $paths=array(
        '/classes/',
        '/classes/core/',
        '/classes/core/exception/',
        '/classes/core/authentication/',
        '/classes/core/interface/',
        '/classes/model/',
        '/XraySourceInterface/',
    );
    foreach($paths as $path){
        if(file_exists(__DIR__.$path.$className.'.class.php')){
            /** @noinspection PhpIncludeInspection */
            include_once(__DIR__.$path.$className.'.class.php');
        }
    }
});


try {
    Log::init(Log::LOGLEVEL_DEBUG);

    database::connect();

    session::init(new DummySession());
    session::becomeAdmin();
    $admin=user::getFirstAdmin();
    $adminId=$admin['id'];
    session::set('userId',$adminId);

    database::begin();

    $group=null;
    try {
        Log::write(Log::LOGLEVEL_INFO, 'Checking for "'.$groupName.'" usergroup...');
        $group=usergroup::getByName($groupName);
        Log::write(Log::LOGLEVEL_INFO, '...exists.');
    } catch (NotFoundException $e){
        Log::write(Log::LOGLEVEL_INFO, 'Usergroup "'.$groupName.'" does not exist, creating...');
        $group=usergroup::create(array(
            'name'=>$groupName,
            'description'=>'Student accounts used for courses',
        ));
        $group=$group['created'];
        Log::write(Log::LOGLEVEL_INFO, '...done.');
    }
    if(!$group){ throw new Exception('Could not find or create usergroup'); }

    for($i=1;$i<=$accounts;$i++){

        $username='student'.$i;
        $password=$username;
        $email=$username.'@bogus.example.com';
        $fullName='Course student ';
        if(10>$i){ $fullName.=' '; } //Extra space ensures alphabetical sort puts users in number order, not 1, 11, 12, 2...
        $fullName.=$i;

        Log::write(Log::LOGLEVEL_INFO, 'Processing student account: '.$fullName);


        $user=user::getByName($username);

        $isInStudentsGroup=false;
        if(!$user){
            Log::write(Log::LOGLEVEL_INFO, 'User does not exist, creating...');
            user::create(array(
                'name'=>$username,
                'password'=>$password,
                'email'=>$email,
                'fullname'=>$fullName,
                'isactive'=>1
            ));
        } else {
            Log::write(Log::LOGLEVEL_INFO, 'User exists, resetting password, etc...');
            user::update($user['id'], array(
                'password'=>$password,
                'email'=>$email,
                'fullname'=>$fullName,
                'isactive'=>1
            ));
            $groups=user::getusergroups($user['id']);
            if($groups){
                $groupNames=array_column($groups['rows'],'name');
            }
            if(in_array($groupName, $groupNames)){
                $isInStudentsGroup=true;
            }
        }

        $user=user::getByName($username);

        if(!$isInStudentsGroup){
            Log::write(Log::LOGLEVEL_INFO, 'Adding to "'.$groupName.'" group');
            groupmembership::create(array(
                'userid'=>$user['id'],
                'usergroupid'=>$group['id']
            ));
            Log::write(Log::LOGLEVEL_INFO, '...done.');
        } else {
            Log::write(Log::LOGLEVEL_INFO, 'User is already in "'.$groupName.'" group');
        }

    }

    database::commit();


} catch (Exception $e){
    database::abort();
    Log::write(Log::LOGLEVEL_ERROR,$e->getMessage() );
    echo $e->getTraceAsString();
}


